-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

local addon, HowMuch = ...
HowMuch.Tooltip = {}

local MARGIN = 4

local function Show(self)
	self:SetLayer(UI.Native.Tooltip:GetLayer())
	local yOffset = MARGIN * 2
	self:SetPoint("TOPRIGHT", UI.Native.Tooltip, "BOTTOMRIGHT", 0, yOffset)
	self:SetVisible(true)
end

local function Hide(self)
	self:SetVisible(false)
end

local function GetCurrencyValues(ccyValue)
	if ccyValue == nil then
		return 0, 0, 0
	end
	
	local plat = math.floor(ccyValue / 10000)
	local remainder = ccyValue - plat * 10000
	local gold = math.floor(remainder / 100)
	local silver = remainder - gold * 100
	return plat, gold, silver
end

local function SetMaxWidth(field, frame1, frame2, frame3, extra)
	local width1 = frame1[field]:GetWidth()
	local width2 = frame2[field]:GetWidth()
	local width3 = frame3[field]:GetWidth()
	local maxWidth = math.max(width1, width2, width3)
	frame1[field]:SetWidth(maxWidth)
	frame2[field]:SetWidth(maxWidth)
	frame3[field]:SetWidth(maxWidth)
	return maxWidth + extra
end

local function ClearMoneyFrameWidths(frame)
	frame.labelText:ClearWidth()
	frame.platText:ClearWidth()
	frame.goldText:ClearWidth()
	frame.silverText:ClearWidth()
	frame:ClearWidth()
end

local function AlignMoneyFrames(widget)
	local width = 0
	width = width + SetMaxWidth("labelText", widget.avgFrame, widget.minFrame, widget.maxFrame, 0)	
	width = width + SetMaxWidth("platText", widget.avgFrame, widget.minFrame, widget.maxFrame, widget.textureWidth)
	width = width + SetMaxWidth("goldText", widget.avgFrame, widget.minFrame, widget.maxFrame, widget.textureWidth)
	width = width + SetMaxWidth("silverText", widget.avgFrame, widget.minFrame, widget.maxFrame, widget.textureWidth)
	widget.avgFrame:SetWidth(width)
	widget.minFrame:SetWidth(width)
	widget.maxFrame:SetWidth(width)
	return width
end

local function SetMoneyFrameValues(widget, value)
	local plat, gold, silver = GetCurrencyValues(value)
	local platVisible = plat > 0
	widget.platText:SetVisible(platVisible)
	widget.platIcon:SetVisible(platVisible)
	
	local goldVisible = (gold > 0) or platVisible
	widget.goldText:SetVisible(goldVisible)
	widget.goldIcon:SetVisible(goldVisible)
	
	widget.platText:SetText(tostring(plat))
	widget.goldText:SetText(tostring(gold))
	widget.silverText:SetText(tostring(silver))
end

local function SetValues(self, avgValue, minValue, maxValue)
	ClearMoneyFrameWidths(self.avgFrame)
	ClearMoneyFrameWidths(self.minFrame)
	ClearMoneyFrameWidths(self.maxFrame)
	
	SetMoneyFrameValues(self.avgFrame, avgValue)
	SetMoneyFrameValues(self.minFrame, minValue)
	SetMoneyFrameValues(self.maxFrame, maxValue)
	
	local width = AlignMoneyFrames(self)
	self:SetWidth(width + 14)
end

local function CreateMoneyFrame(name, parent, label)
	local widget = UI.CreateFrame("Frame", name, parent)
	widget:SetBackgroundColor(0, 0, 0, 1)
	widget:SetHeight(20)
	widget:SetWidth(160)

	widget.labelText = UI.CreateFrame("Text", name .. "labelText", widget)
	widget.silverIcon = UI.CreateFrame("Texture", name .. "silverIcon", widget)
	widget.silverText = UI.CreateFrame("Text", name .. "silverText", widget)
	widget.goldIcon = UI.CreateFrame("Texture", name .. "goldIcon", widget)
	widget.goldText = UI.CreateFrame("Text", name .. "goldText", widget)
	widget.platIcon = UI.CreateFrame("Texture", name .. "platIcon", widget)
	widget.platText = UI.CreateFrame("Text", name .. "platText", widget)
	widget.labelText:SetPoint("TOPLEFT", widget, "TOPLEFT")
	widget.platText:SetPoint("BOTTOMLEFT", widget.labelText, "BOTTOMRIGHT", 0, 0)
	widget.platIcon:SetPoint("BOTTOMLEFT", widget.platText, "BOTTOMRIGHT", 0, 0)
	widget.goldText:SetPoint("BOTTOMLEFT", widget.platIcon, "BOTTOMRIGHT", 0, 0)
	widget.goldIcon:SetPoint("BOTTOMLEFT", widget.goldText, "BOTTOMRIGHT", 0, 0)
	widget.silverText:SetPoint("BOTTOMLEFT", widget.goldIcon, "BOTTOMRIGHT", 0, 0)
	widget.silverIcon:SetPoint("BOTTOMLEFT", widget.silverText, "BOTTOMRIGHT", 0, 0)
	
	widget.labelText:SetText(label)
	widget.platIcon:SetTexture(HowMuch.name, "plat.png")
	widget.goldIcon:SetTexture(HowMuch.name, "gold.png")
	widget.silverIcon:SetTexture(HowMuch.name, "silver.png")
	return widget
end

function HowMuch.Tooltip.Create(name, parent)
	local widget = UI.CreateFrame("Frame", name, parent)
	Library.LibSimpleWidgets.SetBorder("tooltip", widget)
	widget.__lsw_border:SetPosition("inside")
	widget:SetVisible(false)

	widget.avgFrame = CreateMoneyFrame(name .. "avgFrame", widget, "avg")
	widget.avgFrame:SetPoint("TOPLEFT", widget, "TOPLEFT", MARGIN * 2, MARGIN * 2)
	widget.minFrame = CreateMoneyFrame(name .. "avgFrame", widget, "min")
	widget.minFrame:SetPoint("TOPLEFT", widget.avgFrame, "BOTTOMLEFT", 0, 0)
	widget.maxFrame = CreateMoneyFrame(name .. "minFrame", widget, "max")
	widget.maxFrame:SetPoint("TOPLEFT", widget.minFrame, "BOTTOMLEFT", 0, 0)
	widget:SetHeight(widget.avgFrame:GetHeight() + widget.minFrame:GetHeight() + widget.maxFrame:GetHeight() + 16)
	widget:SetWidth(166)
	widget.textureWidth = widget.avgFrame.platIcon:GetWidth()

	widget.Show = Show
	widget.Hide = Hide
	widget.SetValues = SetValues

	return widget
end
